@@ -1,43 +1,61 @@
-# SPDX-License-Identifier: BSD-2-Clause
-
-# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
+# Copyright (c) 2022, Riverbank Computing Limited
+# All rights reserved.
+#
+# This copy of SIP is licensed for use under the terms of the SIP License
+# Agreement.  See the file LICENSE for more details.
+#
+# This copy of SIP may also used under the terms of the GNU General Public
+# License v2 or v3 as published by the Free Software Foundation which can be
+# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 
 from functools import partial
 import os
 
-from ...exceptions import deprecated, UserException
-from ...module import get_latest_version
+from ply import lex, yacc
+
+from ...bindings_configuration import get_bindings_configuration
+from ...exceptions import UserException
 
-from ..bindings_configuration import get_bindings_configuration
 from ..error_log import ErrorLog
 from ..instantiations import instantiate_class
 from ..python_slots import invalid_global_slot, slot_name_detail_map
-from ..scoped_name import ScopedName
 from ..specification import (AccessSpecifier, Argument, ArgumentType,
-        ArrayArgument, CachedName, ClassKey, CodeBlock, Constructor,
-        DocstringFormat, DocstringSignature, EnumBaseType, GILAction,
-        IfaceFile, IfaceFileType, KwArgs, MappedType, Member, Module, Overload,
-        PyQtMethodSpecifier, PySlot, Qualifier, QualifierType, Signature,
+        ArrayArgument, ClassKey, CodeBlock, Constructor, DocstringFormat,
+        DocstringSignature, EnumBaseType, GILAction, IfaceFile, IfaceFileType,
+        KwArgs, MappedType, Member, Module, Overload, PyQtMethodSpecifier,
+        PySlot, Qualifier, QualifierType, ScopedName, Signature,
         SourceLocation, Specification, Transfer, TypeHints, WrappedClass,
         WrappedException, WrappedEnum, WrappedEnumMember)
 from ..templates import encoded_template_name, same_template_signature
+from ..type_hints import get_type_hint
 from ..utils import (argument_as_str, cached_name, find_iface_file,
         normalised_scoped_name, same_base_type)
 
 from . import rules
 from . import tokens
 from .annotations import InvalidAnnotation, validate_annotation_value
-from .ply import lex, yacc
+
 
 class ParserManager:
     """ This object manages the actual lexer and parser objects providing them
     with state and utility functions.
     """
 
-    def __init__(self, hex_version, encoding, target_abi, tags,
-            disabled_features, protected_is_public, include_dirs, sip_module,
-            is_strict):
+    def __init__(self, hex_version, encoding, abi_version, tags,
+            disabled_features, protected_is_public, include_dirs, strict):
         """ Initialise the manager. """
 
         # Create the lexer.
@@ -52,20 +70,19 @@ class ParserManager:
         rules.parser = self._parser
 
         # The list of class templates.  Each element is a 2-tuple of the
-        # template arguments (as a Signature instance) and the class itself.
+        # template arguments and the class itself.
         self.class_templates = []
 
         # Public state.
         self.tags = tags
 
-        self.spec = Specification(target_abi, is_strict, sip_module)
-
-        # The module is initially unnamed.
-        self.modules = [self.spec.module]
+        self.spec = Specification(
+                tuple([int(v) for v in abi_version.split('.')]), strict)
 
         self.c_bindings = None
         self.code_block = None
         self.module_state = None
+        self.module_states = []
         self.paren_depth = 0
         self.parsing_template = False
         self.parsing_virtual = False
@@ -180,7 +197,7 @@ class ParserManager:
                 else:
                     klass.default_ctor = last_resort
 
-            klass.deprecated = annotations.get('Deprecated')
+            klass.deprecated = annotations.get('Deprecated', False)
 
             if klass.convert_to_type_code is not None and annotations.get('AllowNone', False):
                 klass.handles_none = True
@@ -256,7 +273,7 @@ class ParserManager:
                 type_hints=self.get_type_hints(p, symbol, annotations))
 
         klass.class_key = class_key
-        klass.superclasses = superclasses if superclasses is not None else []
+        klass.superclasses = superclasses
 
         self.push_scope(klass,
                 AccessSpecifier.PRIVATE if class_key is ClassKey.CLASS else AccessSpecifier.PUBLIC)
@@ -488,7 +505,7 @@ class ParserManager:
 
         ctor.docstring = docstring
         ctor.gil_action = self._get_gil_action(p, symbol, annotations)
-        ctor.deprecated = annotations.get('Deprecated')
+        ctor.deprecated = annotations.get('Deprecated', False)
 
         if access_specifier is not AccessSpecifier.PRIVATE:
             ctor.kw_args = self._get_kw_args(p, symbol, annotations,
@@ -511,6 +528,11 @@ class ParserManager:
 
         ctor.transfer = self.get_transfer(p, symbol, annotations)
 
+        # /Transfer/ arguments need the wrapper.
+        for ad in py_signature.args:
+            if ad.transfer is Transfer.TRANSFER:
+                ad.get_wrapper = True
+
         scope.ctors.append(ctor)
 
     def add_dtor(self, p, symbol, name, annotations, *, exceptions,
@@ -565,8 +587,7 @@ class ParserManager:
         if self.parsing_virtual or len(scope.dealloc_code) != 0:
             scope.needs_shadow = True
 
-    def add_enum(self, p, symbol, cpp_name, is_scoped, enum_base_type,
-            annotations, members):
+    def add_enum(self, p, symbol, cpp_name, is_scoped, annotations, members):
         """ Create a new enum and add it to the current scope. """
 
         if self.scope_access_specifier is AccessSpecifier.PRIVATE:
@@ -580,9 +601,7 @@ class ParserManager:
         base_type = EnumBaseType.ENUM
 
         if base_type_s is not None:
-            # The minor version of the target ABI may not be known yet (and we
-            # don't need it) so just test the major version.
-            if self.spec.target_abi is not None and self.spec.target_abi[0] < 13:
+            if self.spec.abi_version < (13, 0):
                 self.parser_error(p, symbol,
                         "/BaseType/ is only supported for ABI v13.0 and later")
 
@@ -593,7 +612,7 @@ class ParserManager:
             elif base_type_s == 'IntEnum':
                 base_type = EnumBaseType.INT_ENUM
             elif base_type_s == 'UIntEnum':
-                base_type = EnumBaseType.UINT_ENUM
+                base_type = EnumBaseType.UNSIGNED_INT_ENUM
             elif base_type_s == 'IntFlag':
                 base_type = EnumBaseType.INT_FLAG
             else:
@@ -619,8 +638,7 @@ class ParserManager:
 
         w_enum = WrappedEnum(base_type, fq_cpp_name, self.module_state.module,
                 cached_fq_cpp_name=cached_fq_cpp_name, is_scoped=is_scoped,
-                enum_base_type=enum_base_type, py_name=py_name,
-                scope=self.scope)
+                py_name=py_name, scope=self.scope)
 
         if self.scope_access_specifier is AccessSpecifier.PROTECTED:
             if not self._protected_is_public:
@@ -731,7 +749,7 @@ class ParserManager:
 
         overload.gil_action = self._get_gil_action(p, symbol, annotations)
         overload.factory = annotations.get('Factory', False)
-        overload.deprecated = annotations.get('Deprecated')
+        overload.deprecated = annotations.get('Deprecated', False)
         overload.new_thread = annotations.get('NewThread', False)
         overload.transfer = self.get_transfer(p, symbol, annotations)
 
@@ -799,10 +817,6 @@ class ParserManager:
                 len_method_code = CodeBlock("Auto-generated",
                         text='            sipRes = (Py_ssize_t)sipCpp->{0}();\n'.format(cpp_name))
 
-                # Note that we can't currently use any %MethodCode because it
-                # does too much (ie. setting sipRes).
-                self.scope.len_cpp_name = cpp_name
-
             len_py_signature = Signature(result=Argument(ArgumentType.SSIZE))
 
             self._add_auto_slot(p, symbol, annotations, '__len__',
@@ -921,7 +935,6 @@ class ParserManager:
         """ Apply annotations to a mapped type. """
 
         mapped_type.handles_none = annotations.get('AllowNone', False)
-        mapped_type.movable = annotations.get('Movable', False)
         mapped_type.no_assignment_operator = annotations.get(
                 'NoAssignmentOperator', False)
         mapped_type.no_copy_ctor = annotations.get('NoCopyCtor', False)
@@ -929,11 +942,6 @@ class ParserManager:
         mapped_type.no_release = annotations.get('NoRelease', False)
         mapped_type.type_hints = self.get_type_hints(p, symbol, annotations)
 
-        if mapped_type.no_release:
-            mapped_type.no_assignment_operator = True
-            mapped_type.no_copy_ctor = True
-            mapped_type.no_default_ctor = True
-
         pyqt_flags = self._get_plugin_annotation(p, symbol, annotations,
                 'PyQtFlags', 'PyQt6')
         if pyqt_flags is not None:
@@ -1183,12 +1191,6 @@ class ParserManager:
         # Return any value of the right type.
         return KwArgs.OPTIONAL
 
-    def deprecated(self, p, symbol, instead=None):
-        """ Issue a deprecation message about a symbol. """
-
-        deprecated(f"'{p[symbol]}'", instead=instead, filename=self._sip_file,
-                line_nr=p.lineno(symbol))
-
     def ensure_import(self):
         """ We allow %Modules that are part of a %CompositeModule to be either
         %Imported or %Included.  In the case of the latter we need to adjust
@@ -1198,9 +1200,14 @@ class ParserManager:
         sip_file, raw_sip_file, input, lineno, lexpos, module_state = self._file_stack.pop()
 
         if module_state is None:
-            self._import_module(self._sip_file)
             module_state = self.module_state
 
+            # The current file was %Included so create a new Module and
+            # ModuleState as if it had been %Imported.
+            module = Module()
+            self.spec.modules.append(module)
+            self.module_state = ModuleState(module, self._sip_file, self)
+
         self._file_stack.append(
                 (sip_file, raw_sip_file, input, lineno, lexpos, module_state))
 
@@ -1249,7 +1256,7 @@ class ParserManager:
         if upper_name:
             upper_qual = self._find_timeline_qualifier(p, symbol_upper)
             if upper_qual is None:
-                return True
+                return False
         else:
             upper_qual = None
 
@@ -1292,7 +1299,7 @@ class ParserManager:
 
             return True
 
-        # See if there is a selected qualifier within range.
+        # See if there is a selected qualifier withing range.
         for qual in module.qualifiers:
             if qual.type is QualifierType.TIME and qual.timeline == timeline and qual.name in self.tags:
                 if lower_qual is not None and qual.order < lower_qual.order:
@@ -1308,7 +1315,7 @@ class ParserManager:
     def find_qualifier(self, p, symbol, name, required=True):
         """ Return a Qualifier or None if one doesn't exist. """
 
-        for module in self.modules:
+        for module in self.spec.modules:
             for qual in module.qualifiers:
                 if qual.name == name:
                     return qual
@@ -1410,27 +1417,31 @@ class ParserManager:
         None if none were specified.
         """
 
-        th = annotations.get('TypeHint')
-        th_in = annotations.get('TypeHintIn')
-        th_out = annotations.get('TypeHintOut')
+        th_text = annotations.get('TypeHint')
+        th_in_text = annotations.get('TypeHintIn')
+        th_out_text = annotations.get('TypeHintOut')
         th_value = annotations.get('TypeHintValue')
 
-        if th_in is None:
-            th_in = th
-        elif th is not None:
+        if th_in_text is None:
+            th_in_text = th_text
+        elif th_text is not None:
             self.parser_error(p, symbol,
                     "'TypeHint' and 'TypeHintIn' cannot both be specified")
 
             return None
 
-        if th_out is None:
-            th_out = th
-        elif th is not None:
+        th_in = None if th_in_text is None else get_type_hint(self.spec, th_in_text)
+
+        if th_out_text is None:
+            th_out_text = th_text
+        elif th_text is not None:
             self.parser_error(p, symbol,
                     "'TypeHint' and 'TypeHintOut' cannot both be specified")
 
             return None
 
+        th_out = None if th_out_text is None else get_type_hint(self.spec, th_out_text)
+
         if th_in is not None or th_out is not None or th_value is not None:
             # Check that type hints haven't been suppressed.
             if annotations.get('NoTypeHint') is not None:
@@ -1447,12 +1458,14 @@ class ParserManager:
         code will be generated for.
         """
 
-        return self.module_state.module is self.spec.module
+        module = self.module_state.module
+
+        return module is self.spec.modules[0] or module.composite is not None
 
     def instantiate_class_template(self, p, symbol, fq_cpp_name, template,
             py_name, no_type_name, docstring):
-        """ Try and instantiate a class template and return the instantiated
-        class or None if no template was found.
+        """ Try and instantiate a class template and return True if one was
+        found.
         """
 
         # Look for an appropriate class template.
@@ -1461,10 +1474,12 @@ class ParserManager:
                 break
         else:
             # There was no class template to instantiate.
-            return None
+            return False
 
-        return instantiate_class(p, symbol, fq_cpp_name, tmpl_names,
-                proto_class, template, py_name, no_type_name, docstring, self)
+        instantiate_class(p, symbol, fq_cpp_name, tmpl_names, proto_class,
+                template, py_name, no_type_name, docstring, self)
+
+        return True
 
     def lexer_error(self, t, text):
         """ Record an error caused by a token. """
@@ -1474,10 +1489,9 @@ class ParserManager:
                         column=self._get_column_from_lexpos(t.lexpos)))
 
     def parse(self, sip_file):
-        """ Parse a .sip file and return a 3-tuple of a Specification object, a
-        list of Module objects and a list of the .sip files that specify the
-        module to be generated.  A UserException is raised if there was an
-        error.
+        """ Parse a .sip file and return a Specification object and a list of
+        the .sip files that specify the module to be generated.  A
+        UserException is raised if there was an error.
         """
 
         # Note that the retention of the 'raw' filename, ie. that which was
@@ -1487,7 +1501,7 @@ class ParserManager:
         raw_sip_file = sip_file
         sip_file = os.path.abspath(sip_file)
 
-        self.module_state = ModuleState(self.spec.module, sip_file)
+        self.module_state = ModuleState(self.spec.modules[0], sip_file, self)
 
         try:
             self._parser.parse(self._read(sip_file, raw_sip_file),
@@ -1526,21 +1540,12 @@ class ParserManager:
         for klass in self._template_arg_classes:
             self.spec.classes.remove(klass)
 
-        # Finalise the target ABI version.
-        self._finalise_target_abi()
-
-        return self.spec, self.modules, self._sip_files
+        return self.spec, self._sip_files
 
     def parser_error(self, p, symbol, text):
         """ Record an error caused by a symbol in a production. """
 
-        self.parser_error_at_location(self.get_source_location(p, symbol),
-                text)
-
-    def parser_error_at_location(self, source_location, text):
-        """ Record an error caused by a symbol in a production. """
-
-        self._error_log.log(text, source_location)
+        self._error_log.log(text, self.get_source_location(p, symbol))
 
     def pop_file(self):
         """ Restore the current .sip file from the stack and make it current.
@@ -1630,17 +1635,47 @@ class ParserManager:
                         "'{0}' is being read recursively".format(sip_file))
                 return
 
-        # Ignore the file if we have already read it.
-        if sip_file in self._all_sip_files:
-            return
-
         if new_module:
-            old_module_state = self.module_state
-            self._import_module(sip_file)
+            importing_from = self.module_state.module
+
+            # Create a new module if it has not already been defined.
+            for module_state in self.module_states:
+                if module_state.sip_file == sip_file:
+                    module = module_state.module
+                    break
+            else:
+                module = Module()
+                self.spec.modules.append(module)
+
+                module.default_exception = self.module_state.module.default_exception
+                old_module_state = self.module_state
+                self.module_state = ModuleState(module, sip_file, self)
+
+                # Get the configuration of the new module.
+                mod_tags, mod_disabled = get_bindings_configuration(
+                        self.spec.abi_version[0], sip_file, self._include_dirs)
+
+                for tag in mod_tags:
+                    if tag not in self.tags:
+                        self.tags.append(tag)
+
+                for feature in mod_disabled:
+                    if feature not in self._disabled_features:
+                        self._disabled_features.append(feature)
+
+            # Add the new import unless it has already been imported.
+            if module not in importing_from.imports:
+                importing_from.imports.insert(0, module)
         else:
             # This means that the file was %Included rather than %Imported.
             old_module_state = None
 
+        # Ignore the file if we have already read it. This replicates the
+        # behaviour of the old parser but shouldn't be required for a well
+        # specified project.
+        if sip_file in self._all_sip_files:
+            return
+
         # Save the state of the current .sip file.
         self._file_stack.append(
                 (self._sip_file, self.raw_sip_file, self._input,
@@ -1783,7 +1818,7 @@ class ParserManager:
     def validate_mapped_type(self, p, symbol, mapped_type):
         """ Validate a completed mapped type. """
 
-        if self.spec.target_abi is None or self.spec.target_abi >= (13, 0):
+        if self.spec.abi_version >= (13, 0):
             convert_to_us = mapped_type.convert_to_type_code is not None and 'sipUserState' in mapped_type.convert_to_type_code.text
 
             release_us = mapped_type.release_code is not None and 'sipUserState' in mapped_type.release_code.text
@@ -1859,11 +1894,6 @@ class ParserManager:
 
         for arg in signature.args:
             if arg.type is ArgumentType.ELLIPSIS:
-                # Give the argument the standard name so that it appears in
-                # docstrings and type hints.  It is never used in generated
-                # code so there is no need to add it to the module cache.
-                arg.name = CachedName('*args')
-
                 if seen_ellipsis:
                     self.parser_error(p, symbol,
                             "'...' may only be specified once")
@@ -1888,37 +1918,6 @@ class ParserManager:
                 fq_cpp_name, iface_file_type, error_logger, cpp_type=cpp_type,
                 scope=self.scope)
 
-    def _finalise_target_abi(self):
-        """ Finalise the target ABI. """
-
-        target_abi = self.spec.target_abi
-
-        if target_abi is None:
-            major_version = get_latest_version()
-            minor_version = None
-        else:
-            major_version, minor_version = target_abi
-
-        if minor_version is None:
-            minor_version = get_latest_version(major_version)
-
-        # These ABI versions are deprecated because we have deprecated any
-        # arguments to 'throw()' which these versions rely on.
-        if major_version == 12 and minor_version < 9:
-            self._deprecated_target_abi(major_version, minor_version, '12.9')
-
-        if major_version == 13 and minor_version < 1:
-            self._deprecated_target_abi(major_version, minor_version, '13.1')
-
-        self.spec.target_abi = (major_version, minor_version)
-
-    @staticmethod
-    def _deprecated_target_abi(major_version, minor_version, instead):
-        """ Issue a deprecation warning about an old ABI version. """
-
-        deprecated(f"ABI v{major_version}.{minor_version}",
-                instead=f"v{instead} or later")
-
     def _find_member(self, p, symbol, py_name, args, annotations, method_code):
         """ Return (creating if necessary) the member for a Python name. """
 
@@ -2006,7 +2005,7 @@ class ParserManager:
 
         name = p[symbol]
 
-        qual = self.find_qualifier(p, symbol, name, required=False)
+        qual = self.find_qualifier(p, symbol, name)
         if qual is None:
             return None
 
@@ -2115,34 +2114,6 @@ class ParserManager:
         # call_super_init defaults to False if it wasn't specified.
         module.call_super_init = bool(module_state.call_super_init)
 
-    def _import_module(self, sip_file):
-        """ Create a new Module object and corresponding ModuleState object for
-        a .sip file and make it current.
-        """
-
-        importing_from = self.module_state.module
-
-        module = Module()
-        self.modules.append(module)
-
-        module.default_exception = self.module_state.module.default_exception
-        self.module_state = ModuleState(module, sip_file)
-
-        # Get the configuration of the new module.
-        mod_tags, mod_disabled = get_bindings_configuration(self.spec,
-                sip_file, self._include_dirs)
-
-        for tag in mod_tags:
-            if tag not in self.tags:
-                self.tags.append(tag)
-
-        for feature in mod_disabled:
-            if feature not in self._disabled_features:
-                self._disabled_features.append(feature)
-
-        # Add the new import.
-        importing_from.imports.append(module)
-
     def _read(self, sip_file, raw_sip_file):
         """ Return the contents of the current .sip file. """
 
@@ -2176,7 +2147,7 @@ class ParserManager:
 class ModuleState:
     """ Encapsulate the parser-related state for a module. """
 
-    def __init__(self, module, sip_file):
+    def __init__(self, module, sip_file, pm):
         """ Initialise the state. """
 
         self.module = module
@@ -2189,6 +2160,8 @@ class ModuleState:
         self.kw_args = KwArgs.NONE
         self.nr_timelines = 0
 
+        pm.module_states.append(self)
+
 
 class ScopeState:
     """ Encapsulate the parser-related state for a scope. """
