@@ -1,26 +1,44 @@
-# SPDX-License-Identifier: BSD-2-Clause
-
-# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>
+# Copyright (c) 2022, Riverbank Computing Limited
+# All rights reserved.
+#
+# This copy of SIP is licensed for use under the terms of the SIP License
+# Agreement.  See the file LICENSE for more details.
+#
+# This copy of SIP may also used under the terms of the GNU General Public
+# License v2 or v3 as published by the Free Software Foundation which can be
+# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 
 from dataclasses import dataclass, field
-from enum import auto, Enum
+from enum import Enum
 from typing import Any, Dict, List, Optional, Union
 
-from .scoped_name import ScopedName
-
 
 class AccessSpecifier(Enum):
     """ The class access specifiers. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # Private access.
-    PRIVATE = auto()
+    PRIVATE = 0x04
 
     # Protected access.
-    PROTECTED = auto()
+    PROTECTED = 0x02
 
     # Public access.
-    PUBLIC = auto()
+    PUBLIC = 0x01
 
 
 class ArgumentType(Enum):
@@ -207,38 +225,45 @@ class ArgumentType(Enum):
 class ArrayArgument(Enum):
     """ The array support provided by an argument. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # /Array/ was specified.
-    ARRAY = auto()
+    ARRAY = 0
 
     # /ArraySize/ was specified.
-    ARRAY_SIZE = auto()
+    ARRAY_SIZE = 1
 
     # The argument provides no array support.
-    NONE = auto()
+    NONE = 2
 
 
 class ClassKey(Enum):
     """ The key that identifies a particular type of class. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # A class.
-    CLASS = auto()
+    CLASS = 0
 
     # A struct.
-    STRUCT = auto()
+    STRUCT = 1
 
     # A union.
-    UNION = auto()
+    UNION = 2
 
 
 class DocstringFormat(Enum):
     """ The formatting applied to the text of the docstring. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
+    # The signature is appended to the docstring.
     # Any leading spaces common to all non-blank lines in the docstring are
     # removed.
-    DEINDENTED = auto()
+    DEINDENTED = 1
 
     # The docstring is used as it is specified in the .sip file.
-    RAW = auto()
+    RAW = 0
 
 
 class DocstringSignature(Enum):
@@ -247,318 +272,513 @@ class DocstringSignature(Enum):
     docstring then it applies to all the class's ctors.
     """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # The signature is appended to the docstring.
-    APPENDED = auto()
+    APPENDED = 2
 
     # The signature is discard.
-    DISCARDED = auto()
+    DISCARDED = 0
 
     # The signature is prepended to the docstring.
-    PREPENDED = auto()
+    PREPENDED = 1
 
 
 class EnumBaseType(Enum):
     """ The different base types fo an enum. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # enum.Enum
-    ENUM = auto()
+    ENUM = 0
 
     # enum.Flag
-    FLAG = auto()
+    FLAG = 1
 
     # enum.IntEnum
-    INT_ENUM = auto()
+    INT_ENUM = 2
 
     # enum.IntFlag
-    INT_FLAG = auto()
+    INT_FLAG = 3
 
     # enum.IntEnum with unsigned values.
-    UINT_ENUM = auto()
+    UNSIGNED_INT_ENUM = 4
 
 
 class GILAction(Enum):
     """ The action to take with the GIL when calling C/C++ code. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # The default action.
-    DEFAULT = auto()
+    DEFAULT = 0
 
     # Hold the GIL.
-    HOLD = auto()
+    HOLD = 1
 
     # Release the GIL.
-    RELEASE = auto()
+    RELEASE = 2
 
 
 class IfaceFileType(Enum):
     """ The type of an interface file. """
 
-    # A class.
-    CLASS = auto()
+    # TODO: Change the values to auto() once the C code has been replaced.
 
     # An %Exception.
-    EXCEPTION = auto()
+    EXCEPTION = 0
 
     # A %MappedType.
-    MAPPED_TYPE = auto()
+    MAPPED_TYPE = 1
 
     # A namespace.
-    NAMESPACE = auto()
+    NAMESPACE = 2
+
+    # A class.
+    CLASS = 3
 
 
 class KwArgs(Enum):
     """ The level of support for passing argument as keyword arguments. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # All named arguments can be passed as keyword arguments.
-    ALL = auto()
+    ALL = 1
 
     # Keyword arguments are not supported.
-    NONE = auto()
+    NONE = 0
 
     # All named optional arguments (ie. those with a default value) can be
     # passed as keyword arguments.
-    OPTIONAL = auto()
+    OPTIONAL = 2
 
 
 class PyQtMethodSpecifier(Enum):
     """ The PyQt-specific method specifier. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # A signal.
-    SIGNAL = auto()
+    SIGNAL = 0x10
 
     # A slot.
-    SLOT = auto()
+    SLOT = 0x08
 
 
 class PySlot(Enum):
     """ The Python slots corresponding to entries in a type object. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # tp_str
-    STR = auto()
+    STR = 0
 
     # tp_as_number.nb_int
-    INT = auto()
+    INT = 1
 
     # tp_as_number.nb_float
-    FLOAT = auto()
+    FLOAT = 2
 
     # tp.as_mapping.mp_length and tp.as_sequence.sq_length
-    LEN = auto()
+    LEN = 3
 
     # tp.as_sequence.sq_contains
-    CONTAINS = auto()
+    CONTAINS = 4
 
     # tp_as_number.nb_add
-    ADD = auto()
+    ADD = 5
 
     # tp.as_sequence.sq_concat
-    CONCAT = auto()
+    CONCAT = 6
 
     # tp_as_number.nb_subtract
-    SUB = auto()
+    SUB = 7
 
     # tp_as_number.nb_multiply
-    MUL = auto()
+    MUL = 8
 
     # tp.as_sequence.sq_repeat
-    REPEAT = auto()
+    REPEAT = 9
 
     # tp_as_number.nb_remainder
-    MOD = auto()
+    MOD = 11
 
     # tp_as_number.nb_floor_divide
-    FLOORDIV = auto()
+    FLOORDIV = 12
 
     # tp_as_number.nb_true_divide
-    TRUEDIV = auto()
+    TRUEDIV = 13
 
     # tp_as_number.nb_and
-    AND = auto()
+    AND = 14
 
     # tp_as_number.nb_or
-    OR = auto()
+    OR = 15
 
     # tp_as_number.nb_xor
-    XOR = auto()
+    XOR = 16
 
     # tp_as_number.nb_lshift
-    LSHIFT = auto()
+    LSHIFT = 17
 
     # tp_as_number.nb_rshift
-    RSHIFT = auto()
+    RSHIFT = 18
 
     # tp_as_number.nb_inplace_add
-    IADD = auto()
+    IADD = 19
 
     # tp.as_sequence.sq_inplace_concat
-    ICONCAT = auto()
+    ICONCAT = 20
 
     # tp_as_number.nb_inplace_subtract
-    ISUB = auto()
+    ISUB = 21
 
     # tp_as_number.nb_inplace_multiply
-    IMUL = auto()
+    IMUL = 22
 
     # tp.as_sequence.sq_inplace_repeat
-    IREPEAT = auto()
+    IREPEAT = 23
 
     # tp_as_number.nb_inplace_remainder
-    IMOD = auto()
+    IMOD = 25
 
     # tp_as_number.nb_inplace_floor_divide
-    IFLOORDIV = auto()
+    IFLOORDIV = 26
 
     # tp_as_number.nb_inplace_true_divide
-    ITRUEDIV = auto()
+    ITRUEDIV = 27
 
     # tp_as_number.nb_inplace_and
-    IAND = auto()
+    IAND = 28
 
     # tp_as_number.nb_inplace_or
-    IOR = auto()
+    IOR = 29
 
     # tp_as_number.nb_inplace_xor
-    IXOR = auto()
+    IXOR = 30
 
     # tp_as_number.nb_inplace_lshift
-    ILSHIFT = auto()
+    ILSHIFT = 31
 
     # tp_as_number.nb_inplace_rshift
-    IRSHIFT = auto()
+    IRSHIFT = 32
 
     # tp_as_number.nb_invert
-    INVERT = auto()
+    INVERT = 33
 
     # tp_call
-    CALL = auto()
+    CALL = 34
 
     # tp.as_mapping.mp_subscript and tp.as_sequence.sq_item
-    GETITEM = auto()
+    GETITEM = 35
 
     # tp.as_mapping.mp_ass_subscript and tp.as_sequence.sq_ass_item
-    SETITEM = auto()
+    SETITEM = 36
 
     # tp.as_mapping.mp_ass_subscript and tp.as_sequence.sq_ass_item
-    DELITEM = auto()
+    DELITEM = 37
 
     # tp_richcompare (Py_LT)
-    LT = auto()
+    LT = 38
 
     # tp_richcompare (Py_LE)
-    LE = auto()
+    LE = 39
 
     # tp_richcompare (Py_EQ)
-    EQ = auto()
+    EQ = 40
 
     # tp_richcompare (Py_NE)
-    NE = auto()
+    NE = 41
 
     # tp_richcompare (Py_GT)
-    GT = auto()
+    GT = 42
 
     # tp_richcompare (Py_GE)
-    GE = auto()
+    GE = 43
 
     # tp_as_number.nb_bool
-    BOOL = auto()
+    BOOL = 45
 
     # tp_as_number.nb_negative
-    NEG = auto()
+    NEG = 46
 
     # tp_as_number.nb_positive
-    POS = auto()
+    POS = 47
 
     # tp_as_number.nb_absolute
-    ABS = auto()
+    ABS = 48
 
     # tp_repr
-    REPR = auto()
+    REPR = 49
 
     # tp_hash
-    HASH = auto()
+    HASH = 50
 
     # tp_as_number.nb_index
-    INDEX = auto()
+    INDEX = 51
 
     # tp_iter
-    ITER = auto()
+    ITER = 52
 
     # tp_iter_next
-    NEXT = auto()
+    NEXT = 53
 
     # tp_setattr
-    SETATTR = auto()
+    SETATTR = 54
 
     # Internal to the parser (implemented as tp_setattr)
-    DELATTR = auto()
+    DELATTR = 55
 
     # tp_as_number.nb_matrix_multiply
-    MATMUL = auto()
+    MATMUL = 56
 
     # tp_as_number.nb_inplace_matrix_multiply
-    IMATMUL = auto()
+    IMATMUL = 57
 
     # tp_as_async.am_await
-    AWAIT = auto()
+    AWAIT = 58
 
     # tp_as_async.am_aiter
-    AITER = auto()
+    AITER = 59
 
     # tp_as_async.am_anext
-    ANEXT = auto()
+    ANEXT = 60
 
 
 class QualifierType(Enum):
     """ The type of a qualifier used in %If/%End directives. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # The qualifier is a feature.
-    FEATURE = auto()
+    FEATURE = 2
 
     # The qualifier is a platform.
-    PLATFORM = auto()
+    PLATFORM = 1
 
     # The qualifier is part of a timeline.
-    TIME = auto()
+    TIME = 0
 
 
 class Transfer(Enum):
     """ The different types of ownership transfer. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # No transfer of ownership.
-    NONE = auto()
+    NONE = 0
 
     # /Transfer/ was specified.
-    TRANSFER = auto()
+    TRANSFER = 1
 
     # /TransferBack/ was specified.
-    TRANSFER_BACK = auto()
+    TRANSFER_BACK = 2
 
     # /TransferThis/ was specified.
-    TRANSFER_THIS = auto()
+    TRANSFER_THIS = 3
 
 
 class ValueType(Enum):
     """ The different types of a value in an expression. """
 
+    # TODO: Change the values to auto() once the C code has been replaced.
+
     # A quoted character.
-    QCHAR = auto()
+    QCHAR = 0
 
     # A string.
-    STRING = auto()
+    STRING = 1
 
     # A number.
-    NUMERIC = auto()
+    NUMERIC = 2
 
     # A floating point number.
-    REAL = auto()
+    REAL = 3
 
     # A scoped name.
-    SCOPED = auto()
+    SCOPED = 4
 
     # A function call.
-    FCALL = auto()
+    FCALL = 5
 
     # A placeholder.
-    EMPTY = auto()
+    EMPTY = 6
+
+
+class ScopedName:
+    """ Encapsulate a scoped name. """
+
+    def __init__(self, name):
+        """ Initialise the scoped name. """
+
+        if isinstance(name, list):
+            # This use is internal to this class.
+            self._name = name
+        elif isinstance(name, ScopedName):
+            # Make a copy of another ScopedName object.
+            self._name = list(name._name)
+        else:
+            # Create a simple name.
+            self._name = [name]
+
+    def __eq__(self, other):
+        """ Compare with another scoped name for equality. """
+
+        if isinstance(other, ScopedName):
+            return self._name == other._name
+
+        return NotImplemented
+
+    def __delitem__(self, index):
+        """ Remove the requested name. """
+
+        del self._name[self._normalised_index(index)]
+
+    def __getitem__(self, index):
+        """ Get the requested name. """
+
+        return self._name[self._normalised_index(index)]
+
+    def __len__(self):
+        """ Return the length of the name, ie. the number of indivual names.
+        """
+
+        nr_names = len(self._name)
+
+        if self.is_absolute:
+            nr_names -= 1
+
+        return nr_names
+
+    def __lt__(self, other):
+        """ Compare with another scoped name to allow lists to be sorted. """
+
+        if isinstance(other, ScopedName):
+            return self._name < other._name
+
+        return NotImplemented
+
+    def __setitem__(self, index, name):
+        """ Set the requested name. """
+
+        self._name[self._normalised_index(index)] = name
+
+    def __str__(self):
+        """ Return the C++ string representation. """
+
+        # The special treatment is a hack to simplify how the result is used.
+        # We ignore any leading '::' and truncate the conversion if we think we
+        # are converting an encoded template name (ie. we find a word that
+        # starts with a digit).
+        s_l = []
+        for s in self:
+            if s[0].isdigit():
+                break
+
+            s_l.append(s)
+
+        return '::'.join(s_l)
+
+    @property
+    def absolute(self):
+        """ Return an absolute version of the name. """
+
+        if self.is_absolute:
+            return self
+
+        copy = type(self)(self)
+        copy.make_absolute()
+
+        return copy
+
+    def append(self, name):
+        """ Append a simple name. """
+
+        self._name.append(name)
+
+    @property
+    def base_name(self):
+        """ The base name of the scoped name. """
+
+        return self._name[-1]
+
+    @property
+    def is_absolute(self):
+        """ True if the scoped name is absolute. """
+
+        return self._name[0] == ''
+
+    @property
+    def is_simple(self):
+        """ Return True if the name is simple, ie. relative and unscoped. """
+
+        return len(self) == 1
+
+    def matches(self, scoped_name, scope=None):
+        """ Return True if a scoped name matches this taking account of an
+        optional scope if the scoped name is relative.
+        """
+
+        assert self.is_absolute
+
+        # Check for the simple case.
+        if scoped_name.is_absolute:
+            return self == scoped_name
+
+        # Try each scope, inner to outer.
+        while scope is not None:
+            fq_name = ScopedName(scope.iface_file.fq_cpp_name)
+            fq_name._name.extend(scoped_name._name)
+
+            if self == fq_name:
+                return True
+
+            scope = scope.scope
+
+        # Treat the name as if it were absolute.
+        return self._name[1:] == scoped_name._name
+
+    def make_absolute(self):
+        """ Make sure the scoped name is absolute. """
+
+        if self._name[0] != '':
+            self._name.insert(0, '')
+
+    @classmethod
+    def parse(cls, raw):
+        """ Return a ScopedName object by parsing a raw string. """
+
+        return cls(raw.split('::'))
+
+    def prepend(self, scoped_name):
+        """ Prepend a scoped name. """
+
+        new_name = list(scoped_name._name)
+        new_name.extend(self._name)
+        self._name = new_name
+
+    @property
+    def scope(self):
+        """ The scoped name that is the enclosing scope of this one.  It will
+        be None if there isn't one.
+        """
+
+        if len(self) == 1:
+            return None
+
+        return type(self)(self._name[:-1])
+
+    def _normalised_index(self, index):
+        """ Return a normalised index. """
+
+        if not isinstance(index, int):
+            raise TypeError("'{}' is an invalid index".format(type(index)))
+
+        if index >= 0 and self.is_absolute:
+            index += 1
+
+        return index
 
 
 @dataclass
@@ -632,11 +852,6 @@ class Argument:
     # The non-default type hints.
     type_hints: Optional['TypeHints'] = None
 
-    def __hash__(self):
-        """ Reimplemented so an Argument object can be used as a dict key. """
-
-        return id(self)
-
 
 @dataclass
 class CachedName:
@@ -722,7 +937,7 @@ class Constructor:
     # Set if a Python exception is raised.
     raises_py_exception: bool = False
 
-    # The optional throw arguments.  Replace with 'noexcept' in SIP v7.
+    # The optional throw arguments.
     throw_args: Optional['ThrowArguments'] = None
 
     # Any transfer of ownership.
@@ -750,8 +965,8 @@ class Extract:
     # The order.  A negative value implies the part is appended to the extract.
     order: int
 
-    # The text of the extract part.
-    text: str
+    # The actual part of the extract.
+    part: CodeBlock
 
 
 @dataclass
@@ -917,9 +1132,6 @@ class Member:
     # Set if /Sequence/ was specified.
     is_sequence: bool = False
 
-    # The number of the member. (outputter)
-    member_nr: int = -1
-
     # The original interface file if the function was defined in a namespace.
     namespace_iface_file: Optional[IfaceFile] = None
 
@@ -940,6 +1152,9 @@ class Module:
     # Set if wrapped ctors should support cooperative multi-inheritance.
     call_super_init: bool = False
 
+    # The containing composite module.
+    composite: Optional['Module'] = None
+
     # The text specified by any %Copying directives.
     copying: List[CodeBlock] = field(default_factory=list)
 
@@ -981,6 +1196,9 @@ class Module:
     # The code specified by any %InitialisationCode directives.
     initialisation_code: List[CodeBlock] = field(default_factory=list)
 
+    # Set if the module is a composite module.
+    is_composite: bool = False
+
     # The software license.
     license: Optional[License] = None
 
@@ -1017,9 +1235,6 @@ class Module:
     # The code specified by any %PreInitialisationCode directives.
     preinitialisation_code: List[CodeBlock] = field(default_factory=list)
 
-    # The name of the module. (resolver)
-    py_name: Optional[str] = None
-
     # Set if the generated bindings are Py_ssize_t clean.
     py_ssize_t_clean: bool = False
 
@@ -1056,7 +1271,7 @@ class Overload:
     # The member common to all overloads.
     common: Member
 
-    # The C/C++ name if not a operator/slot.
+    # The C/C++ name.
     cpp_name: str
 
     # The C/C++ signature.
@@ -1152,7 +1367,7 @@ class Overload:
     # The source location.
     source_location: Optional['SourceLocation'] = None
 
-    # The optional throw arguments.  Replace with 'noexcept' in SIP v7.
+    # The optional throw arguments.
     throw_args: Optional['ThrowArguments'] = None
 
     # Any transfer of ownership.
@@ -1216,7 +1431,7 @@ class Signature:
     # The list of arguments.
     args: List[Argument] = field(default_factory=list)
 
-    # The type of the result.
+    # The type of the optional result.
     result: Optional[Argument] = None
 
 
@@ -1244,10 +1459,6 @@ class Specification:
     # Set if the specification is strict.
     is_strict: bool
 
-    # The fully qualified name of the sip module.  If it is None then there is
-    # no shared sip module.
-    sip_module: Optional[str]
-
     # Set if the bindings are for C rather than C++.
     c_bindings: bool = False
 
@@ -1272,17 +1483,14 @@ class Specification:
     # The interface files.
     iface_files: List[IfaceFile] = field(default_factory=list)
 
-    # Set if the specification is for a composite module.
-    is_composite: bool = False
-
     # The mapped type templates.
     mapped_type_templates: List[MappedTypeTemplate] = field(default_factory=list)
 
     # The mapped types.
     mapped_types: List[MappedType] = field(default_factory=list)
 
-    # The module for which code is to be generated.
-    module: Module = field(default_factory=Module)
+    # The list of modules.  Initially there is an unnamed module.
+    modules: List[Module] = field(default_factory=lambda: [Module()])
 
     # The cache of names that may be required as strings in the generated code.
     name_cache: Dict[int, List[CachedName]] = field(default_factory=dict)
@@ -1332,19 +1540,27 @@ class ThrowArguments:
     """ Encapsulate the arguments to a C++ throw(). """
 
     # The list of the argument names. If it is None then 'noexcept' was
-    # specified, otherwise there will be at least one argument.
+    # specified.
     arguments: Optional[List['WrappedException']] = None
 
 
+@dataclass
+class TypeHint:
+    """ Encapsulate a PEP 484 type hint. """
+
+    # The text of the hint.
+    text: str
+
+
 @dataclass
 class TypeHints:
     """ Encapsulate a set of PEP 484 type hints for a type. """
 
     # The type hint when used to pass a value into a callable.
-    hint_in: Optional[str]
+    hint_in: Optional[TypeHint]
 
     # The type hint used to return a value from a callable.
-    hint_out: Optional[str]
+    hint_out: Optional[TypeHint]
 
     # The representation of a default value in a type hint.
     default_value: Optional[str]
@@ -1503,7 +1719,7 @@ class WrappedClass:
     # The action required on the GIL.
     dtor_gil_action: GILAction = GILAction.DEFAULT
 
-    # The optional dtor throw arguments.  Replace with 'noexcept' in SIP v7.
+    # The optional dtor throw arguments.
     dtor_throw_args: Optional[ThrowArguments] = None
 
     # The code specified by any dtor %VirtualCatcherCode directive.
@@ -1557,9 +1773,6 @@ class WrappedClass:
     # Set if the class is QObject or a sub-class. (resolver)
     is_qobject: bool = False
 
-    # The C++ name of any overload annotated with __len__.
-    len_cpp_name: Optional[str] = None
-
     # The methods.
     members: List[Member] = field(default_factory=list)
 
@@ -1646,11 +1859,6 @@ class WrappedClass:
     # The visible member functions. (resolver)
     visible_members: List[VisibleMember] = field(default_factory=list)
 
-    def __hash__(self):
-        """ Reimplemented so an Argument object can be used as a dict key. """
-
-        return id(self)
-
 
 @dataclass
 class WrappedEnum:
@@ -1668,7 +1876,7 @@ class WrappedEnum:
     # The cached fully qualified C++ name.
     cached_fq_cpp_name: Optional[CachedName] = None
 
-    # Set if the enum is defined in a protected section.
+    # Set if the enum is defined in the protected section.
     is_protected: bool = False
 
     # Set if the enum is a scoped enum.
@@ -1703,11 +1911,6 @@ class WrappedEnum:
     # The generated type number. (resolver)
     type_nr: int = -1
 
-    def __hash__(self):
-        """ Reimplemented so an Argument object can be used as a dict key. """
-
-        return id(self)
-
 
 @dataclass
 class WrappedEnumMember:
@@ -1746,8 +1949,7 @@ class WrappedException:
     # The base exception if it is defined in the specification.
     defined_base_exception: Optional['WrappedException'] = None
 
-    # The number of the exception (only if a Python exception object is
-    # required. (resolver)
+    # The number of the exception. (resolver)
     exception_nr: int = -1
 
     # Set if this exception is needed by the module for which code is to be
